@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2018 The OpenMx Project
+#   Copyright 2007-2018 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -231,6 +231,7 @@ setMethod("genericExpFunConvert", "MxExpectationGREML",
               .Object@X <- as.matrix(mxDataObject@observed[,-1])
               .Object@yXcolnames <- colnames(mxDataObject@observed)
               .Object@numFixEff <- as.integer(ncol(mxDataObject@observed)-1)
+	      .Object@dataColumnNames <- colnames(mxDataObject@observed)
               .Object@dataColumns <- 0:(nrow(mxDataObject@observed)-1L)
             }
             else{
@@ -259,6 +260,7 @@ setMethod("genericExpFunConvert", "MxExpectationGREML",
               .Object@yXcolnames <- colnames(mm$yX)
               .Object@casesToDrop <- mm$casesToDrop
               .Object@numFixEff <- ncol(.Object@X)
+	      .Object@dataColumnNames <- colnames(.Object@X)
               .Object@dataColumns <- 0:(nrow(.Object@X)-1L)
             }
             #Get number of observed statistics BEFORE call to backend, so summary() can use it:
