@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2018 by the individuals mentioned in the source code history
+#   Copyright 2007-2019 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -18,7 +18,7 @@ mxRObjective <- function(objfun, ...) {
 		stop("First argument 'objfun' must be of type function")
 	}
 	if (length(formals(objfun)) != 2) {
-		stop("The objective function must take exactly two arguments: a model and a persistant state")
+		stop("The objective function must take exactly two arguments: a model and a persistent state")
 	}
 	expectation <- NULL
 	fitfunction <- mxFitFunctionR(objfun, ...)
