@@ -40,35 +40,36 @@ void GradientOptimizerContext::copyBounds()
 	fc->copyBoxConstraintToOptimizer(solLB, solUB);
 }
 
-void GradientOptimizerContext::setupSimpleBounds()
+void GradientOptimizerContext::setupSimpleBounds() //used with SLSQP.
 {
 	solLB.resize(numFree);
 	solUB.resize(numFree);
 	copyBounds();
-}
+	//MxConstraints are re-counted in omxInvokeNLOPT().
+} 
 
-void GradientOptimizerContext::setupIneqConstraintBounds()
+void GradientOptimizerContext::setupIneqConstraintBounds() //used with CSOLNP.
 {
 	solLB.resize(numFree);
 	solUB.resize(numFree);
 	copyBounds();
 
 	omxState *globalState = fc->state;
-	int eqn, nineqn;
-	globalState->countNonlinearConstraints(eqn, nineqn, false);
-	equality.resize(eqn);
-	inequality.resize(nineqn);
+	globalState->countNonlinearConstraints(globalState->numEqC, globalState->numIneqC, false);
+	equality.resize(globalState->numEqC);
+	inequality.resize(globalState->numIneqC);
 };
 
-void GradientOptimizerContext::setupAllBounds()
+void GradientOptimizerContext::setupAllBounds() //used with NPSOL.
 {
 	omxState *st = fc->state;
 	int n = (int) numFree;
 
 	// treat all constraints as non-linear
-	int eqn, nineqn;
-	st->countNonlinearConstraints(eqn, nineqn, false);
-	int ncnln = eqn + nineqn;
+	// st->countNonlinearConstraints(eqn, nineqn, false);
+	// ^^^Does the special handling of linear constraints work properly in NPSOL version 6??
+	st->countNonlinearConstraints(st->numEqC, st->numIneqC, false);
+	int ncnln = st->numEqC + st->numIneqC;
 	solLB.resize(n + ncnln);
 	solUB.resize(n + ncnln);
 
@@ -95,7 +96,7 @@ void GradientOptimizerContext::setupAllBounds()
 			}
 			break;
 		default:
-			Rf_error("Unknown constraint type %d", type);
+			mxThrow("Unknown constraint type %d", type);
 		}
 	}
 }
@@ -117,14 +118,17 @@ int GradientOptimizerContext::countNumFree()
 GradientOptimizerContext::GradientOptimizerContext(FitContext *_fc, int _verbose,
 						   enum GradientAlgorithm _gradientAlgo,
 						   int _gradientIterations,
-						   double _gradientStepSize)
+						   double _gradientStepSize,
+						   omxCompute *owner)
 	: fc(_fc), verbose(_verbose), numFree(countNumFree()),
 	  gradientAlgo(_gradientAlgo), gradientIterations(_gradientIterations),
 	  gradientStepSize(_gradientStepSize),
 	  numOptimizerThreads((fc->childList.size() && !fc->openmpUser)? fc->childList.size() : 1),
+	  equality(fc->equality), inequality(fc->inequality), 
+	  analyticEqJacTmp(fc->analyticEqJacTmp), analyticIneqJacTmp(fc->analyticIneqJacTmp),
 	  gwrContext(numOptimizerThreads, numFree, _gradientAlgo, _gradientIterations, _gradientStepSize)
 {
-	optName = "?";
+	computeName = owner->name;
 	fitMatrix = NULL;
 	ControlMinorLimit = 800;
 	ControlRho = 1.0;
@@ -185,7 +189,7 @@ double GradientOptimizerContext::solFun(double *myPars, int* mode)
 	if (*mode == 1) {
 		fc->iterations += 1;
 		fc->resetOrdinalRelativeError();
-		Global->reportProgress(optName, fc);
+		Global->reportProgress(getOptName(), fc);
 	}
 	copyFromOptimizer(myPars, fc);
 
@@ -196,9 +200,9 @@ double GradientOptimizerContext::solFun(double *myPars, int* mode)
 		fc->grad.setZero();
 		want |= FF_COMPUTE_GRADIENT;
 	}
-	ComputeFit(optName, fitMatrix, want, fc);
+	ComputeFit(getOptName(), fitMatrix, want, fc);
 
-	if (fc->outsideFeasibleSet() || isErrorRaised() || Global->timedOut) {
+	if (fc->outsideFeasibleSet() || isErrorRaised()) {
 		*mode = -1;
 	} else {
 		feasible = true;
@@ -218,99 +222,18 @@ double GradientOptimizerContext::solFun(double *myPars, int* mode)
 // variable group.
 void GradientOptimizerContext::solEqBFun(bool wantAJ) //<--"want analytic Jacobian"
 {
-	const int eq_n = (int) equality.size();
-	omxState *st = fc->state;
-
-	if (!eq_n) return;
-	
-	/*Note that this needs to happen even if no equality constraints have analytic Jacobians, because
-	analyticEqJacTmp is copied to the Jacobian matrix the elements of which are populated by code in
-	finiteDifferences.h, which knows to numerically populate an element if it's NA:*/
-	analyticEqJacTmp.setConstant(NA_REAL);
-	
-	int cur=0, j=0, c=0, roffset=0;
-	for(j = 0; j < int(st->conListX.size()); j++) {
-		omxConstraint &con = *st->conListX[j];
-		if (con.opCode != omxConstraint::EQUALITY) continue;
-		
-		con.refreshAndGrab(fc, &equality(cur));
-		if(wantAJ && usingAnalyticJacobian && con.jacobian != NULL){
-			omxRecompute(con.jacobian, fc);
-			for(c=0; c<con.jacobian->cols; c++){
-				if(con.jacMap[c]<0){continue;}
-				for(roffset=0; roffset<con.size; roffset++){
-					analyticEqJacTmp(cur+roffset,con.jacMap[c]) = con.jacobian->data[c * con.size + roffset];
-				}
-			}
-		}
-		cur += con.size;
-	}
-
-	if (verbose >= 3) {
-		mxPrintMat("equality", equality);
-	}
+	fc->solEqBFun(wantAJ, verbose);
+	return;
 };
 
 // NOTE: All non-linear constraints are applied regardless of free
 // variable group.
 void GradientOptimizerContext::myineqFun(bool wantAJ)
 {
-	const int ineq_n = (int) inequality.size();
-	omxState *st = fc->state;
-
-	if (!ineq_n) return;
-	
-	analyticIneqJacTmp.setConstant(NA_REAL);
-	
-	int cur=0, j=0, c=0, roffset=0;
-	for (j=0; j < int(st->conListX.size()); j++) {
-		omxConstraint &con = *st->conListX[j];
-		if (con.opCode == omxConstraint::EQUALITY) continue;
-		
-		con.refreshAndGrab(fc, (omxConstraint::Type) ineqType, &inequality(cur));
-		if(wantAJ && usingAnalyticJacobian && con.jacobian != NULL){
-			omxRecompute(con.jacobian, fc);
-			for(c=0; c<con.jacobian->cols; c++){
-				if(con.jacMap[c]<0){continue;}
-				for(roffset=0; roffset<con.size; roffset++){
-					analyticIneqJacTmp(cur+roffset,con.jacMap[c]) = con.jacobian->data[c * con.size + roffset];
-				}
-			}
-		}
-		cur += con.size;
-	}
-	
-	if (CSOLNP_HACK) {
-		// CSOLNP doesn't know that inequality constraints can be inactive TODO
-	} else {
-		//SLSQP seems to require inactive inequality constraint functions to be held constant at zero:
-		inequality = inequality.array().max(0.0);
-		if(wantAJ && usingAnalyticJacobian){
-			for(int i=0; i<analyticIneqJacTmp.rows(); i++){
-				/*The Jacobians of each inactive constraint are set to zero here; 
-				as their elements will be zero rather than NaN, the code in finiteDifferences.h will leave them alone:*/
-				if(!inequality[i]){analyticIneqJacTmp.row(i).setZero();}
-			}
-		}
-	}
-
-	if (verbose >= 3) {
-		mxPrintMat("inequality", inequality);
-	}
+	fc->myineqFun(wantAJ, verbose, ineqType, CSOLNP_HACK);
+	return;
 };
 
-void GradientOptimizerContext::checkForAnalyticJacobians()
-{
-	usingAnalyticJacobian = false;
-	omxState *st = fc->state;
-	for(int i=0; i < (int) st->conListX.size(); i++){
-		omxConstraint &cs = *st->conListX[i];
-		if(cs.jacobian){
-			usingAnalyticJacobian = true;
-			return;
-		}
-	}
-}
 
 // ------------------------------------------------------------
 
@@ -335,9 +258,10 @@ class omxComputeGD : public omxCompute {
 	int maxIter;
 
 	bool useGradient;
-	SEXP hessChol;
+
 	int nudge;
 
+	Eigen::MatrixXd hessChol;  // in-out for warmstart
 	int warmStartSize;
 	double *warmStart;
 	int threads;
@@ -356,7 +280,6 @@ class omxCompute *newComputeGradientDescent()
 
 omxComputeGD::omxComputeGD()
 {
-	hessChol = NULL;
 	warmStart = NULL;
 }
 
@@ -387,12 +310,12 @@ void omxComputeGD::initFromFrontend(omxState *globalState, SEXP rObj)
 #if HAS_NPSOL
 		engine = OptEngine_NPSOL;
 #else
-		Rf_error("NPSOL is not available in this build. See ?omxGetNPSOL() to download this optimizer");
+		mxThrow("NPSOL is not available in this build. See ?omxGetNPSOL() to download this optimizer");
 #endif
 	} else if(strEQ(engineName, "SD")){
 		engine = OptEngine_SD;
 	} else {
-		Rf_error("%s: engine %s unknown", name, engineName);
+		mxThrow("%s: engine %s unknown", name, engineName);
 	}
 
 	ScopedProtect p5(slotValue, R_do_slot(rObj, Rf_install("useGradient")));
@@ -413,7 +336,7 @@ void omxComputeGD::initFromFrontend(omxState *globalState, SEXP rObj)
 		int *dimList = INTEGER(matrixDims);
 		int rows = dimList[0];
 		int cols = dimList[1];
-		if (rows != cols) Rf_error("%s: warmStart matrix must be square", name);
+		if (rows != cols) mxThrow("%s: warmStart matrix must be square", name);
 
 		warmStartSize = rows;
 		warmStart = REAL(slotValue);
@@ -442,7 +365,7 @@ void omxComputeGD::initFromFrontend(omxState *globalState, SEXP rObj)
 		} else if (strEQ(gradientAlgoName, "central")) {
 			gradientAlgo = GradientAlgorithm_Central;
 		} else {
-		Rf_error("%s: gradient algorithm '%s' unknown", name, gradientAlgoName);
+		mxThrow("%s: gradient algorithm '%s' unknown", name, gradientAlgoName);
 		}
 	}
 
@@ -477,7 +400,7 @@ void omxComputeGD::computeImpl(FitContext *fc)
 	//if (fc->ciobj) verbose=2;
 	double effectiveGradientStepSize = gradientStepSize;
 	if (engine == OptEngine_NLOPT) effectiveGradientStepSize *= GRADIENT_FUDGE_FACTOR(2.0);
-	GradientOptimizerContext rf(fc, verbose, gradientAlgo, gradientIterations, effectiveGradientStepSize);
+	GradientOptimizerContext rf(fc, verbose, gradientAlgo, gradientIterations, effectiveGradientStepSize,this);
 	threads = rf.numOptimizerThreads;
 	rf.fitMatrix = fitMatrix;
 	rf.ControlTolerance = optimalityTolerance;
@@ -526,11 +449,7 @@ void omxComputeGD::computeImpl(FitContext *fc)
 		rf.finish();
 		fc->wanted |= FF_COMPUTE_GRADIENT;
 		if (rf.hessOut.size() ){
-			if (!hessChol) {
-				Rf_protect(hessChol = Rf_allocMatrix(REALSXP, numParam, numParam));
-			}
-			Eigen::Map<Eigen::MatrixXd> hc(REAL(hessChol), numParam, numParam);
-			hc = rf.hessOut;
+			hessChol = rf.hessOut;
 			Eigen::Map<Eigen::MatrixXd> dest(fc->getDenseHessUninitialized(), numParam, numParam);
 			dest.noalias() = rf.hessOut.transpose() * rf.hessOut;
 			fc->wanted |= FF_COMPUTE_HESSIAN;
@@ -545,7 +464,6 @@ void omxComputeGD::computeImpl(FitContext *fc)
         case OptEngine_CSOLNP:
 		if (rf.maxMajorIterations == -1) rf.maxMajorIterations = Global->majorIterations;
 		rf.CSOLNP_HACK = true;
-		rf.checkForAnalyticJacobians();
 		omxCSOLNP(rf);
 		rf.finish();
 		if (rf.gradOut.size()) {
@@ -561,7 +479,6 @@ void omxComputeGD::computeImpl(FitContext *fc)
 		break;
         case OptEngine_NLOPT:
 		if (rf.maxMajorIterations == -1) rf.maxMajorIterations = Global->majorIterations;
-		rf.checkForAnalyticJacobians();
 		omxInvokeNLOPT(rf);
 		rf.finish();
 		fc->wanted |= FF_COMPUTE_GRADIENT;
@@ -580,13 +497,15 @@ void omxComputeGD::computeImpl(FitContext *fc)
 			omxSD(rf);   // unconstrained problems
 			rf.finish();
 		} else {
-			Rf_error("Constrained problems are not implemented");
+			mxThrow("Constrained problems are not implemented");
 		}
 		fc->wanted |= FF_COMPUTE_GRADIENT;
 		break;}
-        default: Rf_error("Optimizer %d is not available", engine);
+        default: mxThrow("Optimizer %d is not available", engine);
 	}
 
+	if (isErrorRaised()) return;
+
 	if (!fc->insideFeasibleSet()) {
 		fc->setInform(INFORM_STARTING_VALUES_INFEASIBLE);
 	} else {
@@ -599,8 +518,6 @@ void omxComputeGD::computeImpl(FitContext *fc)
 		      name, engineName, fc->getLocalComputeCount() - beforeEval, fc->getInform());
 	}
 
-	if (isErrorRaised()) return;
-
 	// Optimizers can terminate with inconsistent fit and parameters
 	ComputeFit(name, fitMatrix, FF_COMPUTE_FIT, fc);
 
@@ -668,8 +585,8 @@ void omxComputeGD::reportResults(FitContext *fc, MxRList *slots, MxRList *out)
 	}
 	slots->add("output", output.asR());
 	
-	if (engine == OptEngine_NPSOL && hessChol) {
-		out->add("hessianCholesky", hessChol);
+	if (engine == OptEngine_NPSOL && hessChol.size()) {
+		out->add("hessianCholesky", Rcpp::wrap(hessChol));
 	}
 }
 
@@ -755,22 +672,20 @@ void ComputeCI::initFromFrontend(omxState *globalState, SEXP rObj)
 		} else if (strEQ(ctypeName, "none")) {
 			// OK
 		} else {
-			Rf_error("%s: unknown constraintType='%s'", name, ctypeName);
+			mxThrow("%s: unknown constraintType='%s'", name, ctypeName);
 		}
 	}
 
 	fitMatrix = omxNewMatrixFromSlot(rObj, globalState, "fitfunction");
 	omxCompleteFitFunction(fitMatrix);
 
-	pushIndex(NA_INTEGER);
+	PushLoopIndex pli(name, NA_INTEGER);
 
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("plan")));
 	SEXP s4class;
 	Rf_protect(s4class = STRING_ELT(Rf_getAttrib(slotValue, R_ClassSymbol), 0));
 	plan = omxNewCompute(globalState, CHAR(s4class));
 	plan->initFromFrontend(globalState, slotValue);
-
-	popIndex();
 }
 
 extern "C" { void F77_SUB(npoptn)(char* string, int Rf_length); };
@@ -780,10 +695,10 @@ class notImplementedConstraint : public omxConstraint {
 public:
 	notImplementedConstraint() : super("not implemented") {};
 	virtual void refreshAndGrab(FitContext *fc, Type ineqType, double *out) {
-		Rf_error("Not implemented");
+		mxThrow("Not implemented");
 	};
 	virtual omxConstraint *duplicate(omxState *dest) {
-		Rf_error("Not implemented");
+		mxThrow("Not implemented");
 	}
 };
 
@@ -871,7 +786,6 @@ void ComputeCI::recordCI(Method meth, ConfidenceInterval *currentCI, int lower,
 		REAL(VECTOR_ELT(detail, 7+px))[detailRow] = Est[px];
 	}
 	++detailRow;
-	popIndex();
 }
 
 struct regularCIobj : CIobjective {
@@ -910,7 +824,7 @@ struct regularCIobj : CIobjective {
 
 		if (!(want & FF_COMPUTE_FIT)) {
 			if (want & (FF_COMPUTE_PREOPTIMIZE | FF_COMPUTE_INITIAL_FIT)) return;
-			Rf_error("Not implemented yet");
+			mxThrow("Not implemented yet");
 		}
 
 		// We need to compute the fit here because that's the only way to
@@ -988,7 +902,7 @@ struct bound1CIobj : CIobjective {
 
 		if (!(want & FF_COMPUTE_FIT)) {
 			if (want & (FF_COMPUTE_PREOPTIMIZE | FF_COMPUTE_INITIAL_FIT)) return;
-			Rf_error("Not implemented yet");
+			mxThrow("Not implemented yet");
 		}
 
 		omxFitFunctionCompute(fitMat->fitFunction, FF_COMPUTE_FIT, fc);
@@ -1071,7 +985,7 @@ struct boundAwayCIobj : CIobjective {
 
 		if (!(want & FF_COMPUTE_FIT)) {
 			if (want & (FF_COMPUTE_PREOPTIMIZE | FF_COMPUTE_INITIAL_FIT)) return;
-			Rf_error("Not implemented yet");
+			mxThrow("Not implemented yet");
 		}
 
 		omxFitFunctionCompute(fitMat->fitFunction, FF_COMPUTE_FIT, fc);
@@ -1166,7 +1080,7 @@ struct boundNearCIobj : CIobjective {
 
 		if (!(want & FF_COMPUTE_FIT)) {
 			if (want & (FF_COMPUTE_PREOPTIMIZE | FF_COMPUTE_INITIAL_FIT)) return;
-			Rf_error("Not implemented yet");
+			mxThrow("Not implemented yet");
 		}
 
 		omxFitFunctionCompute(fitMat->fitFunction, FF_COMPUTE_FIT, fc);
@@ -1224,7 +1138,7 @@ void ComputeCI::boundAdjCI(FitContext *mle, FitContext &fc, ConfidenceInterval *
 
 	bool boundActive = fabs(Mle[currentCI->varIndex] - nearBox) < sqrt(std::numeric_limits<double>::epsilon());
 	if (currentCI->bound[!side] > 0.0) {	// ------------------------------ away from bound side --
-		pushIndex(detailRow);
+		PushLoopIndex pli(name, detailRow);
 		if (!boundActive) {
 			Diagnostic diag;
 			double val;
@@ -1283,7 +1197,7 @@ void ComputeCI::boundAdjCI(FitContext *mle, FitContext &fc, ConfidenceInterval *
 
  part2:
 	if (currentCI->bound[side] > 0.0) {     // ------------------------------ near to bound side --
-		pushIndex(detailRow);
+		PushLoopIndex pli(name, detailRow);
 		double boundLL = NA_REAL;
 		double sqrtCrit95 = sqrt(currentCI->bound[side]);
 		if (!boundActive) {
@@ -1473,7 +1387,7 @@ void ComputeCI::regularCI2(FitContext *mle, FitContext &fc, ConfidenceInterval *
 		int lower = 1-upper;
 		if (!(currentCI->bound[upper])) continue;
 
-		pushIndex(detailRow);
+		PushLoopIndex pli(name, detailRow);
 		Global->checkpointMessage(mle, mle->est, "%s[%d, %d] %s CI",
 					  matName.c_str(), currentCI->row + 1, currentCI->col + 1,
 					  upper? "upper" : "lower");
@@ -1486,7 +1400,7 @@ void ComputeCI::regularCI2(FitContext *mle, FitContext &fc, ConfidenceInterval *
 
 void ComputeCI::computeImpl(FitContext *mle)
 {
-	if (intervals) Rf_error("Can only compute CIs once");
+	if (intervals) mxThrow("Can only compute CIs once");
 	if (!Global->intervals) {
 		if (verbose >= 1) mxLog(name, "%s: mxRun(..., intervals=FALSE), skipping");
 		return;
@@ -1527,7 +1441,7 @@ void ComputeCI::computeImpl(FitContext *mle)
 			omxMatrix *ciMat = oCI->getMatrix(state);
 			omxRecompute(ciMat, mle);
 			if (!ciMat->isValidElem(oCI->row, oCI->col)) {
-				Rf_error("%s: attempt to find confidence interval of "
+				mxThrow("%s: attempt to find confidence interval of "
 					 "nonexistent element (%d,%d) in %dx%d matrix '%s'",
 					 name, 1+oCI->row, 1+oCI->col, ciMat->rows, ciMat->cols, ciMat->name());
 			}
@@ -1671,7 +1585,7 @@ void ComputeCI::reportResults(FitContext *fc, MxRList *slots, MxRList *out)
 
 class ComputeTryH : public omxCompute {
 	typedef omxCompute super;
-	omxCompute *plan;
+	std::unique_ptr< omxCompute > plan;
 	int numFree;
 	int verbose;
 	double loc;
@@ -1687,8 +1601,6 @@ class ComputeTryH : public omxCompute {
 
 	static bool satisfied(FitContext *fc);
 public:
-	ComputeTryH() : plan(0) {};
-	virtual ~ComputeTryH();
 	virtual void initFromFrontend(omxState *, SEXP rObj);
 	virtual void computeImpl(FitContext *fc);
 	virtual void reportResults(FitContext *fc, MxRList *slots, MxRList *out);
@@ -1698,11 +1610,6 @@ public:
 omxCompute *newComputeTryHard()
 { return new ComputeTryH(); }
 
-ComputeTryH::~ComputeTryH()
-{
-	if (plan) delete plan;
-}
-
 void ComputeTryH::initFromFrontend(omxState *globalState, SEXP rObj)
 {
 	super::initFromFrontend(globalState, rObj);
@@ -1722,16 +1629,14 @@ void ComputeTryH::initFromFrontend(omxState *globalState, SEXP rObj)
 	invocations = 0;
 	numRetries = 0;
 
-	pushIndex(NA_INTEGER);
+	PushLoopIndex pli(name, NA_INTEGER);
 
 	SEXP slotValue;
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("plan")));
 	SEXP s4class;
 	Rf_protect(s4class = STRING_ELT(Rf_getAttrib(slotValue, R_ClassSymbol), 0));
-	plan = omxNewCompute(globalState, CHAR(s4class));
+	plan = std::unique_ptr< omxCompute >(omxNewCompute(globalState, CHAR(s4class)));
 	plan->initFromFrontend(globalState, slotValue);
-
-	popIndex();
 }
 
 bool ComputeTryH::satisfied(FitContext *fc)
@@ -1761,20 +1666,20 @@ void ComputeTryH::computeImpl(FitContext *fc)
 		mxLog("%s: at most %d attempts (Welcome)", name, retriesRemain);
 	}
 
-	pushIndex(1);
-
-	bestStatus = INFORM_UNINITIALIZED;
-	bestFit = NA_REAL;
-	fc->setInform(INFORM_UNINITIALIZED);
-	plan->compute(fc);
-	if (fc->getInform() != INFORM_UNINITIALIZED && fc->getInform() != INFORM_STARTING_VALUES_INFEASIBLE) {
-		bestStatus = fc->getInform();
-		bestEst = curEst;
-		bestFit = fc->fit;
+	{
+		PushLoopIndex pli(name, 1);
+		bestStatus = INFORM_UNINITIALIZED;
+		bestFit = NA_REAL;
+		fc->setInform(INFORM_UNINITIALIZED);
+		plan->compute(fc);
+		if (fc->getInform() != INFORM_UNINITIALIZED &&
+		    fc->getInform() != INFORM_STARTING_VALUES_INFEASIBLE) {
+			bestStatus = fc->getInform();
+			bestEst = curEst;
+			bestFit = fc->fit;
+		}
 	}
 
-	popIndex();
-
 	while (!satisfied(fc) && retriesRemain > 0) {
 		if (verbose >= 2) {
 			mxLog("%s: fit %.2f inform %d, %d retries remain", name, fc->fit,
@@ -1782,21 +1687,22 @@ void ComputeTryH::computeImpl(FitContext *fc)
 		}
 
 		curEst = origStart;
-		GetRNGstate();
-		for (int vx=0; vx < curEst.size(); ++vx) {
-			double adj1 = loc + unif_rand() * 2.0 * scale - scale;
-			double adj2 = 0.0 + unif_rand() * 2.0 * scale - scale;
-			if (verbose >= 3) {
-				mxLog("%d %g %g", vx, adj1, adj2);
+		{
+			BorrowRNGState grs;
+			for (int vx=0; vx < curEst.size(); ++vx) {
+				double adj1 = loc + unif_rand() * 2.0 * scale - scale;
+				double adj2 = 0.0 + unif_rand() * 2.0 * scale - scale;
+				if (verbose >= 3) {
+					mxLog("%d %g %g", vx, adj1, adj2);
+				}
+				curEst[vx] = curEst[vx] * adj1 + adj2;
+				if(curEst[vx] < solLB[vx]){curEst[vx] = solLB[vx];}
+				if(curEst[vx] > solUB[vx]){curEst[vx] = solUB[vx];}
 			}
-			curEst[vx] = curEst[vx] * adj1 + adj2;
-			if(curEst[vx] < solLB[vx]){curEst[vx] = solLB[vx];}
-			if(curEst[vx] > solUB[vx]){curEst[vx] = solUB[vx];}
 		}
-		PutRNGstate();
 
 		--retriesRemain;
-		pushIndex(maxRetries - retriesRemain);
+		PushLoopIndex pli(name, maxRetries - retriesRemain);
 
 		fc->setInform(INFORM_UNINITIALIZED);
 		fc->wanted &= ~(FF_COMPUTE_GRADIENT | FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN);
@@ -1807,7 +1713,6 @@ void ComputeTryH::computeImpl(FitContext *fc)
 			bestEst = curEst;
 			bestFit = fc->fit;
 		}
-		popIndex();
 	}
 
 	fc->setInform(bestStatus);
@@ -1825,7 +1730,7 @@ void ComputeTryH::collectResults(FitContext *fc, LocalComputeResult *lcr, MxRLis
 {
 	super::collectResults(fc, lcr, out);
 	std::vector< omxCompute* > clist(1);
-	clist[0] = plan;
+	clist[0] = plan.get();
 	collectResultsHelper(fc, clist, lcr, out);
 }
 
@@ -1844,6 +1749,7 @@ class ComputeGenSA : public omxCompute {
 	omxCompute *plan;
 	static const char *optName;
 	const char *methodName;
+	std::string contextStr;
 	int numFree;
 	int numIneqC;
 	int numEqC;
@@ -1948,7 +1854,8 @@ void ComputeGenSA::initFromFrontend(omxState *state, SEXP rObj)
 		methodName = R_CHAR(STRING_ELT(Rmethod, 0));
 		if (strEQ(methodName, "tsallis1996")) method = ALGO_TSALLIS1996;
 		else if (strEQ(methodName, "ingber2012")) method = ALGO_INGBER2012;
-		else Rf_error("%s: unknown method '%s'", name, methodName);
+		else mxThrow("%s: unknown method '%s'", name, methodName);
+		contextStr = string_snprintf("%s(%s)", name, methodName);
 
 		ProtectedSEXP Rcontrol(R_do_slot(rObj, Rf_install("control")));
 		ProtectedSEXP RcontrolName(Rf_getAttrib(Rcontrol, R_NamesSymbol));
@@ -2017,11 +1924,11 @@ void ComputeGenSA::initFromFrontend(omxState *state, SEXP rObj)
 					Rf_warning("%s: unknown key '%s' for method '%s'",
 						   name, key, methodName);
 				}
-			} else Rf_error("%s: method %d unimplemented", name, method);
+			} else mxThrow("%s: method %d unimplemented", name, method);
 		}
 	}
 
-	pushIndex(NA_INTEGER);
+	PushLoopIndex pli(name, NA_INTEGER);
 
 	SEXP slotValue;
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("plan")));
@@ -2029,8 +1936,6 @@ void ComputeGenSA::initFromFrontend(omxState *state, SEXP rObj)
 	Rf_protect(s4class = STRING_ELT(Rf_getAttrib(slotValue, R_ClassSymbol), 0));
 	plan = omxNewCompute(state, CHAR(s4class));
 	plan->initFromFrontend(state, slotValue);
-
-	popIndex();
 }
 
 double ComputeGenSA::visita(double temp)
@@ -2104,21 +2009,23 @@ double ComputeGenSA::asa_cost(double *x, int *cost_flag, int *exit_code, USER_DE
 	Map< VectorXd > curEst(fc->est, numFree);
 	Map< VectorXd > proposal(x, numFree);
 
-	PutRNGstate();
-	pushIndex(opt->N_Generated);
-	fc->setInform(INFORM_UNINITIALIZED);
-	curEst = proposal;
-	fc->copyParamToModel();
-	fc->wanted = FF_COMPUTE_FIT;
-	plan->compute(fc);
-	popIndex();
-	GetRNGstate();
+	{
+		ReturnRNGState rrs;
+		PushLoopIndex pli(name, opt->N_Generated);
+		fc->setInform(INFORM_UNINITIALIZED);
+		curEst = proposal;
+		fc->copyParamToModel();
+		fc->wanted = FF_COMPUTE_FIT;
+		plan->compute(fc);
+	}
 
+	if (Global->interrupted()) return nan("abort");
 	if (fc->outsideFeasibleSet()) {
 		return std::numeric_limits<double>::max();
 	}
 	double penalty = getConstraintPenalty(fc);
 	fc->fit += penalty * round(opt->N_Generated / 100); //OK? TODO
+	Global->reportProgress1(contextStr.c_str(), fc->asProgressReport());
 	return fc->fit;
 }
 
@@ -2143,8 +2050,8 @@ void ComputeGenSA::ingber2012(FitContext *fc)
 		quenchParamScale.setConstant(1.);
 	}
 	if (quenchParamScale.size() != numFree) {
-		Rf_error("%s: quenchParamScale must have %d entries instead of %d",
-			 numFree, quenchParamScale.size());
+		mxThrow("%s: quenchParamScale must have %d entries instead of %d",
+			name, numFree, quenchParamScale.size());
 	}
 	OPTIONS->User_Quench_Param_Scale = quenchParamScale.data();
 
@@ -2153,8 +2060,8 @@ void ComputeGenSA::ingber2012(FitContext *fc)
 		quenchCostScale.setConstant(1.);
 	}
 	if (quenchCostScale.size() != numFree) {
-		Rf_error("%s: quenchCostScale must have %d entries instead of %d",
-			 numFree, quenchCostScale.size());
+		mxThrow("%s: quenchCostScale must have %d entries instead of %d",
+			name, numFree, int(quenchCostScale.size()));
 	}
 	OPTIONS->User_Quench_Cost_Scale = quenchCostScale.data();
 
@@ -2163,12 +2070,13 @@ void ComputeGenSA::ingber2012(FitContext *fc)
 	OPTIONS->Asa_Data_Dim_Ptr = 1;
 	OPTIONS->Asa_Data_Ptr = this;
 
-	GetRNGstate();
-	(void) asa(asa_cost_function_stub, asa_random_generator, &seed,
-		   fc->est, lbound.data(), ubound.data(), tangents.data(),
-		   0, &num_parameters, paramType.data(), &valid_state_generated_flag,
-		   &exit_status, OPTIONS);
-	PutRNGstate();
+	{
+		BorrowRNGState grs;
+		(void) asa(asa_cost_function_stub, asa_random_generator, &seed,
+			   fc->est, lbound.data(), ubound.data(), tangents.data(),
+			   0, &num_parameters, paramType.data(), &valid_state_generated_flag,
+			   &exit_status, OPTIONS);
+	}
 
 	if (!valid_state_generated_flag && verbose) mxLog("invalid state generated");
 
@@ -2197,10 +2105,10 @@ void ComputeGenSA::ingber2012(FitContext *fc)
 	case INVALID_USER_INPUT:
 	case INVALID_COST_FUNCTION:
 	case INVALID_COST_FUNCTION_DERIV:
-		Rf_error("%s: ASA error %d", name, exit_status);
+		mxThrow("%s: ASA error %d", name, exit_status);
 		break;
 	case CALLOC_FAILED:
-		Rf_error("%s: out of memory", name);
+		mxThrow("%s: out of memory", name);
 		break;
 	default:
 		Rf_warning("%s: unknown exit_status %d", name, exit_status);
@@ -2223,17 +2131,17 @@ void ComputeGenSA::tsallis1996(FitContext *fc)
 	double curFit = curBestFit;
 	double curPenalty = curBestPenalty;
 
-	GetRNGstate();
+	BorrowRNGState grs;
 	const double t1 = exp((qv - 1.) * M_LN2) - 1.;
 
 	// Tsallis & Stariolo (1996) start at t=1 (see around Eqn 4)
-	for (int tt = 1; !isErrorRaised() && !Global->timedOut; ++tt) {
+	for (int tt = 1; !isErrorRaised(); ++tt) {
 		// Equation 14' from Tsallis & Stariolo (1996)
 		double t2 = exp((qv - 1.) * log(tt + 1.));
 		double tem = temSta * t1 / t2;
 		if (tem < temEnd) break;
 
-		for (int jj = 0; jj < markovLength; ++jj) {
+		for (int jj = 0; jj < markovLength && !isErrorRaised(); ++jj) {
 			int vx = jj % numFree;
 			double va = visita(tem);
 			double a = xMini[vx] + va;
@@ -2246,14 +2154,14 @@ void ComputeGenSA::tsallis1996(FitContext *fc)
 			}
 			curEst[vx] = a;
 
-			PutRNGstate();
-			pushIndex(jj);
-			fc->setInform(INFORM_UNINITIALIZED);
-			fc->copyParamToModel();
-			fc->wanted = FF_COMPUTE_FIT;
-			plan->compute(fc);
-			popIndex();
-			GetRNGstate();
+			{
+				ReturnRNGState rrs;
+				PushLoopIndex pli(name, jj);
+				fc->setInform(INFORM_UNINITIALIZED);
+				fc->copyParamToModel();
+				fc->wanted = FF_COMPUTE_FIT;
+				plan->compute(fc);
+			}
 
 			if (fc->outsideFeasibleSet()) {
 				curEst[vx] = xMini[vx];
@@ -2299,11 +2207,10 @@ void ComputeGenSA::tsallis1996(FitContext *fc)
 				// candidate rejected
 				curEst[vx] = xMini[vx];
 			}
+			Global->reportProgress(contextStr.c_str(), fc);
 		}
 	}
 
-	PutRNGstate();
-
 	curEst = curBest;
 }
 
@@ -2335,15 +2242,17 @@ void ComputeGenSA::computeImpl(FitContext *fc)
 	// ----------
 
 	ComputeFit(optName, fitMatrix, FF_COMPUTE_FIT, fc);
-	GetRNGstate();
-	int retries = 5;
-	while (fc->outsideFeasibleSet() && retries-- > 0) {
-		for (int vx=0; vx < curEst.size(); ++vx) {
-			curEst[vx] = lbound[vx] + unif_rand() * range[vx];
+	{
+		BorrowRNGState grs;
+		int retries = 5;
+		while (fc->outsideFeasibleSet() && retries-- > 0) {
+			for (int vx=0; vx < curEst.size(); ++vx) {
+				curEst[vx] = lbound[vx] + unif_rand() * range[vx];
+			}
+			ComputeFit(optName, fitMatrix, FF_COMPUTE_FIT, fc);
 		}
-		ComputeFit(optName, fitMatrix, FF_COMPUTE_FIT, fc);
 	}
-	PutRNGstate();
+
 	if (fc->outsideFeasibleSet()) {
 		fc->setInform(INFORM_STARTING_VALUES_INFEASIBLE);
 		return;
@@ -2352,7 +2261,7 @@ void ComputeGenSA::computeImpl(FitContext *fc)
 	switch (method) {
 	case ALGO_TSALLIS1996: tsallis1996(fc); break;
 	case ALGO_INGBER2012: ingber2012(fc); break;
-	default: Rf_error("%s: unknown method %d", name, method);
+	default: mxThrow("%s: unknown method %d", name, method);
 	}
 
 	fc->copyParamToModel();
