@@ -35,40 +35,9 @@
 #include "omxNPSOLSpecific.h"
 #include "omxExportBackendState.h"
 #include "Compute.h"
-#include "omxBuffer.h"
+#include "CovEntrywisePar.h"
 #include "EnableWarnings.h"
-
-class omxComputeNumericDeriv : public omxCompute {
-	typedef omxCompute super;
-	double stepSize;
-	int numIter;
-	bool parallel;
-	int totalProbeCount;
-	int verbose;
-	bool wantHessian;
-	bool checkGradient;
-	double *knownHessian;
-	std::vector<int> khMap;
-
-	omxMatrix *fitMat;
-	double minimum;
-	Eigen::ArrayXd optima;
-	int numParams;
-	double *gcentral, *gforward, *gbackward;
-	double *hessian;
-	bool recordDetail;
-	SEXP detail;
-
-	void omxPopulateHessianWork(struct hess_struct *hess_work, FitContext* fc);
-	void omxEstimateHessianOnDiagonal(int i, struct hess_struct* hess_work);
-	void omxEstimateHessianOffDiagonal(int i, int l, struct hess_struct* hess_work);
-	void doHessianCalculation(int numChildren, struct hess_struct *hess_work);
-
- public:
-        virtual void initFromFrontend(omxState *, SEXP rObj);
-        virtual void computeImpl(FitContext *fc);
-        virtual void reportResults(FitContext *fc, MxRList *slots, MxRList *out);
-};
+#include "finiteDifferences.h"
 
 struct hess_struct {
 	int probeCount;
@@ -87,6 +56,72 @@ struct hess_struct {
 	}
 };
 
+class omxComputeNumericDeriv : public omxCompute {
+	typedef omxCompute super;
+	double stepSize;
+	int numIter;
+	bool parallel;
+	int totalProbeCount;
+	int verbose;
+	bool wantHessian;
+	bool checkGradient;
+	double *knownHessian;
+	std::vector<int> khMap;
+
+	omxMatrix *fitMat;
+	double minimum;
+	Eigen::ArrayXd optima;
+	int numParams;
+	std::vector<int> paramMap;
+	double *gcentral, *gforward, *gbackward;
+	double *hessian;
+	struct hess_struct *hessWorkVector;
+	bool recordDetail;
+	SEXP detail;
+
+	void omxPopulateHessianWork(struct hess_struct *hess_work, FitContext* fc);
+	void omxEstimateHessianOnDiagonal(int i, struct hess_struct* hess_work);
+	void omxEstimateHessianOffDiagonal(int i, int l, struct hess_struct* hess_work);
+	void omxCalcFinalConstraintJacobian(FitContext* fc, int npar);
+
+	struct calcHessianEntry {
+		omxComputeNumericDeriv &cnd;
+		const int numParams;
+		double *hessian;
+		struct hess_struct *hessWorkVector;
+		calcHessianEntry(omxComputeNumericDeriv *_cnd) :
+			cnd(*_cnd), numParams(cnd.numParams), hessian(cnd.hessian),
+			hessWorkVector(cnd.hessWorkVector)
+		{};
+		int getNumCols() { return numParams; };
+		bool isDone(int rx, int cx) {
+			if (hessian) {
+				return std::isfinite(hessian[cx*numParams + rx]);
+			} else {
+				return rx != cx;  // Only interested in gradient
+			}
+		}
+		void onDiag(int ii) {
+			int threadId = omx_absolute_thread_num();
+			cnd.omxEstimateHessianOnDiagonal(ii, hessWorkVector + threadId);
+		}
+		void offDiag(int rx, int cx) {
+			int threadId = omx_absolute_thread_num();
+			cnd.omxEstimateHessianOffDiagonal(rx, cx, hessWorkVector + threadId);
+		}
+		void reportProgress(int numDone) {
+			std::string detail = std::to_string(numDone) + "/" +
+				std::to_string(triangleLoc1(numParams));
+			Global->reportProgress1(cnd.name, detail);
+		}
+	};
+	
+ public:
+        virtual void initFromFrontend(omxState *, SEXP rObj);
+        virtual void computeImpl(FitContext *fc);
+        virtual void reportResults(FitContext *fc, MxRList *slots, MxRList *out);
+};
+
 void omxComputeNumericDeriv::omxPopulateHessianWork(struct hess_struct *hess_work, FitContext* fc)
 {
 	hess_work->probeCount = 0;
@@ -96,7 +131,6 @@ void omxComputeNumericDeriv::omxPopulateHessianWork(struct hess_struct *hess_wor
 	hess_work->Gbackward = new double[numIter];
 	hess_work->fitMatrix = fc->lookupDuplicate(fitMat);
 	hess_work->fc = fc;
-	memcpy(fc->est, optima.data(), numParams * sizeof(double));
 }
 
 /**
@@ -108,6 +142,7 @@ void omxComputeNumericDeriv::omxPopulateHessianWork(struct hess_struct *hess_wor
  */
 void omxComputeNumericDeriv::omxEstimateHessianOnDiagonal(int i, struct hess_struct* hess_work)
 {
+	int ix = paramMap[i];
 	static const double v = 2.0; //Note: NumDeriv comments that this could be a parameter, but is hard-coded in the algorithm
 
 	double *Haprox             = hess_work->Haprox;
@@ -121,7 +156,7 @@ void omxComputeNumericDeriv::omxEstimateHessianOnDiagonal(int i, struct hess_str
 	/* Part the first: Gradient and diagonal */
 	double iOffset = std::max(fabs(stepSize * optima[i]), stepSize);
 	for(int k = 0; k < numIter; k++) {			// Decreasing step size, starting at k == 0
-		freeParams[i] = optima[i] + iOffset;
+		freeParams[ix] = optima[i] + iOffset;
 		
 		fc->copyParamToModel();
 
@@ -129,7 +164,7 @@ void omxComputeNumericDeriv::omxEstimateHessianOnDiagonal(int i, struct hess_str
 		omxRecompute(fitMatrix, fc);
 		double f1 = omxMatrixElement(fitMatrix, 0, 0);
 
-		freeParams[i] = optima[i] - iOffset;
+		freeParams[ix] = optima[i] - iOffset;
 
 		fc->copyParamToModel();
 
@@ -141,7 +176,7 @@ void omxComputeNumericDeriv::omxEstimateHessianOnDiagonal(int i, struct hess_str
 		Gforward[k] = (minimum - f2) / iOffset;
 		Gbackward[k] = (f1 - minimum) / iOffset;
 		Haprox[k] = (f1 - 2.0 * minimum + f2) / (iOffset * iOffset);		// This is second derivative
-		freeParams[i] = optima[i];									// Reset parameter value
+		freeParams[ix] = optima[i];									// Reset parameter value
 		iOffset /= v;
 		if(verbose >= 2) {
 			mxLog("Hessian: diag[%s] Δ%g (#%d) F1 %f F2 %f grad %f hess %f",
@@ -170,6 +205,8 @@ void omxComputeNumericDeriv::omxEstimateHessianOnDiagonal(int i, struct hess_str
 
 void omxComputeNumericDeriv::omxEstimateHessianOffDiagonal(int i, int l, struct hess_struct* hess_work)
 {
+	int ix = paramMap[i];
+	int lx = paramMap[l];
     static const double v = 2.0; //Note: NumDeriv comments that this could be a parameter, but is hard-coded in the algorithm
 
 	double *Haprox             = hess_work->Haprox;
@@ -181,8 +218,8 @@ void omxComputeNumericDeriv::omxEstimateHessianOffDiagonal(int i, int l, struct
 	double lOffset = std::max(fabs(stepSize*optima[l]), stepSize);
 
 	for(int k = 0; k < numIter; k++) {
-		freeParams[i] = optima[i] + iOffset;
-		freeParams[l] = optima[l] + lOffset;
+		freeParams[ix] = optima[i] + iOffset;
+		freeParams[lx] = optima[l] + lOffset;
 
 		fc->copyParamToModel();
 
@@ -190,8 +227,8 @@ void omxComputeNumericDeriv::omxEstimateHessianOffDiagonal(int i, int l, struct
 		omxRecompute(fitMatrix, fc);
 		double f1 = omxMatrixElement(fitMatrix, 0, 0);
 
-		freeParams[i] = optima[i] - iOffset;
-		freeParams[l] = optima[l] - lOffset;
+		freeParams[ix] = optima[i] - iOffset;
+		freeParams[lx] = optima[l] - lOffset;
 
 		fc->copyParamToModel();
 
@@ -207,8 +244,8 @@ void omxComputeNumericDeriv::omxEstimateHessianOffDiagonal(int i, int l, struct
 			      v, k, pow(v, k), stepSize*optima[i], stepSize*optima[l]);
 		}
 
-		freeParams[i] = optima[i];				// Reset parameter values
-		freeParams[l] = optima[l];
+		freeParams[ix] = optima[i];				// Reset parameter values
+		freeParams[lx] = optima[l];
 
 		iOffset = iOffset / v;					//  And shrink step
 		lOffset = lOffset / v;
@@ -230,60 +267,14 @@ void omxComputeNumericDeriv::omxEstimateHessianOffDiagonal(int i, int l, struct
 	hessian[l*numParams+i] = Haprox[0];
 }
 
-void omxComputeNumericDeriv::doHessianCalculation(int numChildren, struct hess_struct *hess_work)
-{
-	// gcc does not detect the usage of the following variable
-	// in the omp parallel pragma, and marks the variable as
-	// unused, so the attribute is placed to silence the Rf_warning.
-    int __attribute__((unused)) parallelism = (numChildren == 0) ? 1 : numChildren;
-
-	std::vector<std::pair<int,int> > todo;
-	if (wantHessian) {
-		todo.reserve(numParams * (numParams-1) / 2);
-		for(int i = 0; i < numParams; i++) {
-			for(int j = i - 1; j >= 0; j--) {
-				if (std::isfinite(hessian[i*numParams + j])) continue;
-				todo.push_back(std::make_pair(i,j));
-			}
-		}
-	}
-
-	if (numChildren) {
-#pragma omp parallel for num_threads(parallelism)
-		for(int i = 0; i < numParams; i++) {
-			if (hessian && std::isfinite(hessian[i*numParams + i])) continue;
-			int threadId = (numChildren < 2) ? 0 : omx_absolute_thread_num();
-			omxEstimateHessianOnDiagonal(i, hess_work + threadId);
-		}
-
-		reportProgress(hess_work->fc);
-
-#pragma omp parallel for num_threads(parallelism)
-		for(int i = 0; i < int(todo.size()); i++) {
-			int threadId = (numChildren < 2) ? 0 : omx_absolute_thread_num();
-			omxEstimateHessianOffDiagonal(todo[i].first, todo[i].second, hess_work + threadId);
-		}
-	} else {
-		for(int i = 0; i < numParams; i++) {
-			reportProgress(hess_work->fc);
-			if (hessian && std::isfinite(hessian[i*numParams + i])) continue;
-			omxEstimateHessianOnDiagonal(i, hess_work);
-		}
-		for(int i = 0; i < int(todo.size()); i++) {
-			reportProgress(hess_work->fc);
-			omxEstimateHessianOffDiagonal(todo[i].first, todo[i].second, hess_work);
-		}
-	}
-}
-
 void omxComputeNumericDeriv::initFromFrontend(omxState *state, SEXP rObj)
 {
 	super::initFromFrontend(state, rObj);
 
-	if (state->conListX.size()) {
-		Rf_error("Cannot compute estimated Hessian with constraints (%d constraints found)",
-		      state->conListX.size());
-	}
+	/*if (state->conListX.size()) {
+		mxThrow("%s: cannot proceed with constraints (%d constraints found)",
+			name, int(state->conListX.size()));
+	}*/
 
 	fitMat = omxNewMatrixFromSlot(rObj, state, "fitfunction");
 
@@ -291,7 +282,7 @@ void omxComputeNumericDeriv::initFromFrontend(omxState *state, SEXP rObj)
 
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("iterations")));
 	numIter = INTEGER(slotValue)[0];
-	if (numIter < 2) Rf_error("stepSize must be 2 or greater");
+	if (numIter < 2) mxThrow("stepSize must be 2 or greater");
 
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("parallel")));
 	parallel = Rf_asLogical(slotValue);
@@ -309,7 +300,7 @@ void omxComputeNumericDeriv::initFromFrontend(omxState *state, SEXP rObj)
 
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("stepSize")));
 	stepSize = GRADIENT_FUDGE_FACTOR(3.0) * REAL(slotValue)[0];
-	if (stepSize <= 0) Rf_error("stepSize must be positive");
+	if (stepSize <= 0) mxThrow("stepSize must be positive");
 
 	knownHessian = NULL;
 	{
@@ -347,6 +338,30 @@ void omxComputeNumericDeriv::initFromFrontend(omxState *state, SEXP rObj)
 	detail = 0;
 }
 
+void omxComputeNumericDeriv::omxCalcFinalConstraintJacobian(FitContext* fc, int npar){
+	allconstraints_functional acf(*fc, verbose);
+	Eigen::MatrixWrapper< Eigen::ArrayXd > optimaM(optima);
+	Eigen::VectorXd resulttmp(fc->state->numEqC + fc->state->numIneqC);
+	Eigen::MatrixXd jactmp(fc->state->numEqC + fc->state->numIneqC, npar);
+	acf(optimaM, resulttmp, jactmp);
+	/*Gradient algorithm, iterations, and stepsize are hardcoded as they are for two reasons.
+	 * 1.  Differentiating the constraint functions should not take long, expecially compared to 
+	 * twice-differentiating the fitfunction, so it might as well be done carefully.
+	 * 2.  The default behavior during the ComputeNumericDeriv step uses different values of 
+	 * gradient stepsize and iterations depending on whether or not the MxModel contains thresholds,
+	 * since the numerical accuracy of the -2logL is worse when multivariate-normal integration is involved.
+	 * But, that has no bearing on the constraint functions, so it doesn't really make sense to use
+	 * the stepsize and iterations stored in the omxComputeNumericDeriv object.
+	 */
+	fd_jacobian<true>(
+		GradientAlgorithm_Central, 4, 1.0e-7,
+    acf, resulttmp, optimaM, jactmp);
+	
+	fc->constraintFunVals = resulttmp;
+	fc->constraintJacobian = jactmp;
+	return;
+}
+
 void omxComputeNumericDeriv::computeImpl(FitContext *fc)
 {
 	if (fc->fitUnits == FIT_UNITS_SQUARED_RESIDUAL ||
@@ -354,47 +369,54 @@ void omxComputeNumericDeriv::computeImpl(FitContext *fc)
 		numParams = 0;
 		if (verbose >= 1) mxLog("%s: derivatives %s units are meaningless",
 					name, fitUnitsToName(fc->fitUnits));
-		return;
+		return; //Possible TODO: calculate Hessian anyway?
 	}
 
 	int newWanted = fc->wanted | FF_COMPUTE_GRADIENT;
 	if (wantHessian) newWanted |= FF_COMPUTE_HESSIAN;
 
-	if (numParams != 0 && numParams != int(fc->numParam)) {
-		Rf_error("%s: number of parameters changed from %d to %d",
-			 name, numParams, int(fc->numParam));
+	int nf = fc->calcNumFree();
+	if (numParams != 0 && numParams != nf) {
+		mxThrow("%s: number of parameters changed from %d to %d",
+			 name, numParams, nf);
 	}
 
-	numParams = int(fc->numParam);
+	numParams = nf;
 	if (numParams <= 0) { complainNoFreeParam(); return; }
 
 	optima.resize(numParams);
-	memcpy(optima.data(), fc->est, sizeof(double) * numParams);
+	fc->copyEstToOptimizer(optima);
+	paramMap.resize(numParams);
+	for (int px=0,ex=0; px < numParams; ++ex) {
+		if (fc->profiledOut[ex]) continue;
+		paramMap[px++] = ex;
+	}
 
 	omxAlgebraPreeval(fitMat, fc);
 	fc->createChildren(fitMat); // allow FIML rowwiseParallel even when parallel=false
 
-	if(wantHessian && fc->state->conListX.size()){
-		Rf_warning("due to presence of MxConstraints, Hessian matrix and standard errors may not be valid for statistical-inferential purposes");
+	fc->state->countNonlinearConstraints(fc->state->numEqC, fc->state->numIneqC, false);
+	int c_n = fc->state->numEqC + fc->state->numIneqC;
+	fc->constraintFunVals.resize(c_n);
+	fc->constraintJacobian.resize(c_n, numParams);
+	if(c_n){
+		omxCalcFinalConstraintJacobian(fc, numParams);
 	}
-	// TODO: Adjust algorithm to account for constraints
 	// TODO: Allow more than one hessian value for calculation
 
-	int numChildren = 0;
-	if (parallel && !fc->openmpUser) numChildren = fc->childList.size();
+	int numChildren = 1;
+	if (parallel && !fc->openmpUser && fc->childList.size()) numChildren = fc->childList.size();
 
-	if (!fc->haveReferenceFit(fitMat) || Global->timedOut) return;
+	if (!fc->haveReferenceFit(fitMat)) return;
 
 	minimum = fc->fit;
 
-	struct hess_struct* hess_work;
-	if (numChildren < 2) {
-		hess_work = new hess_struct[1];
-		omxPopulateHessianWork(hess_work, fc);
+	hessWorkVector = new hess_struct[numChildren];
+	if (numChildren == 1) {
+		omxPopulateHessianWork(hessWorkVector, fc);
 	} else {
-		hess_work = new hess_struct[numChildren];
 		for(int i = 0; i < numChildren; i++) {
-			omxPopulateHessianWork(hess_work + i, fc->childList[i]);
+			omxPopulateHessianWork(hessWorkVector + i, fc->childList[i]);
 		}
 	}
 	if(verbose >= 1) mxLog("Numerical Hessian approximation (%d children, ref fit %.2f)",
@@ -453,17 +475,15 @@ void omxComputeNumericDeriv::computeImpl(FitContext *fc)
 	Gc.setConstant(NA_REAL);
 	Gb.setConstant(NA_REAL);
 
-	doHessianCalculation(numChildren, hess_work);
+	calcHessianEntry che(this);
+	CovEntrywiseParallel(numChildren, che);
 
-	if (numChildren < 2) {
-		totalProbeCount = hess_work->probeCount;
-	} else {
-		for(int i = 0; i < numChildren; i++) {
-			struct hess_struct *hw = hess_work + i;
-			totalProbeCount += hw->probeCount;
-		}
+	for(int i = 0; i < numChildren; i++) {
+		struct hess_struct *hw = hessWorkVector + i;
+		totalProbeCount += hw->probeCount;
 	}
-	delete [] hess_work;
+	delete [] hessWorkVector;
+	if (isErrorRaised()) return;
 
 	Eigen::Map< Eigen::ArrayXi > Gsymmetric(LOGICAL(VECTOR_ELT(detail, 0)), numParams);
 	double gradNorm = 0.0;
@@ -471,7 +491,7 @@ void omxComputeNumericDeriv::computeImpl(FitContext *fc)
 	double feasibilityTolerance = Global->feasibilityTolerance;
 	for (int px=0; px < numParams; ++px) {
 		// factor out simliar code in ComputeNR
-		omxFreeVar &fv = *fc->varGroup->vars[px];
+		omxFreeVar &fv = *fc->varGroup->vars[ paramMap[px] ];
 		if ((fabs(optima[px] - fv.lbound) < feasibilityTolerance && Gc[px] > 0) ||
 		    (fabs(optima[px] - fv.ubound) < feasibilityTolerance && Gc[px] < 0)) {
 			Gsymmetric[px] = false;
@@ -485,20 +505,28 @@ void omxComputeNumericDeriv::computeImpl(FitContext *fc)
 		}
 	}
 	
-	fc->grad.resize(numParams);
-	fc->grad = Gc.matrix();
+	fc->grad.resize(fc->numParam);
+	fc->grad.setZero();
+	fc->copyGradFromOptimizer(Gc);
+	
+	if(c_n){
+		fc->inequality.resize(fc->state->numIneqC);
+		fc->analyticIneqJacTmp.resize(fc->state->numIneqC, numParams);
+		fc->myineqFun(true, verbose, omxConstraint::LESS_THAN, false);
+	}
 
 	gradNorm = sqrt(gradNorm);
 	double gradThresh = Global->getGradientThreshold(minimum);
-	if (checkGradient && gradNorm > gradThresh) {
+	//The gradient will generally not be near zero at a local minimum if there are equality constraints 
+	//or active inequality constraints:
+	if ( checkGradient && gradNorm > gradThresh && !(fc->state->numEqC || fc->inequality.array().sum()) ) {
 		if (verbose >= 1) {
 			mxLog("Some gradient entries are too large, norm %f", gradNorm);
 		}
 		if (fc->getInform() < INFORM_NOT_AT_OPTIMUM) fc->setInform(INFORM_NOT_AT_OPTIMUM);
 	}
 
-	memcpy(fc->est, optima.data(), sizeof(double) * numParams);
-	fc->copyParamToModel();
+	fc->setEstFromOptimizer(optima);
 	// auxillary information like per-row likelihoods need a refresh
 	ComputeFit(name, fitMat, FF_COMPUTE_FIT, fc);
 	fc->wanted = newWanted;
@@ -506,7 +534,7 @@ void omxComputeNumericDeriv::computeImpl(FitContext *fc)
 
 void omxComputeNumericDeriv::reportResults(FitContext *fc, MxRList *slots, MxRList *result)
 {
-	if (!numParams || !(fc->wanted & (FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN))) return;
+	if (!numParams || !(fc->wanted & (FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN | FF_COMPUTE_GRADIENT))) return;
 
 	if (wantHessian) {
 		SEXP calculatedHessian;
@@ -518,7 +546,7 @@ void omxComputeNumericDeriv::reportResults(FitContext *fc, MxRList *slots, MxRLi
 	MxRList out;
 	out.add("probeCount", Rf_ScalarInteger(totalProbeCount));
 	if (detail && recordDetail) {
-		Eigen::Map< Eigen::ArrayXi > Gsymmetric(LOGICAL(VECTOR_ELT(detail, 0)), fc->numParam);
+		Eigen::Map< Eigen::ArrayXi > Gsymmetric(LOGICAL(VECTOR_ELT(detail, 0)), numParams);
 		out.add("gradient", detail);
 	}
 	slots->add("output", out.asR());
