@@ -16,6 +16,7 @@
 
 #include "omxDefines.h"
 
+#include <utility>
 #include <stan/math/mix/mat.hpp>
 #include "multi_normal_sufficient.hpp"
 
@@ -258,7 +259,7 @@ void MLFitState::compute(int want, FitContext *fc)
 			delete hb;
 		}
 	} else {
-		Rf_error("Not implemented");
+		mxThrow("Not implemented");
 	}
 }
 
@@ -306,7 +307,7 @@ omxFitFunction *MLFitState::initMorph()
 {
 	auto *oo = this;
 
-	if (!oo->expectation) { Rf_error("%s requires an expectation", oo->fitType); }
+	if (!oo->expectation) { mxThrow("%s requires an expectation", oo->fitType); }
 	oo->units = FIT_UNITS_MINUS2LL;
 
 	if (strcmp(expectation->expType, "MxExpectationBA81")==0) {
@@ -337,11 +338,11 @@ omxFitFunction *MLFitState::initMorph()
 				strEQ(oo->expectation->expType, "MxExpectationRAM") && !wantRowwiseLikelihood);
 
 	if (Rf_asLogical(Rfellner) == 1 && !fellnerPossible) {
-		Rf_error("%s: fellner requires raw data (have %s), "
+		mxThrow("%s: fellner requires raw data (have %s), "
 			 "all continuous indicators (%d are ordinal), "
 			 "MxExpectationRAM (have %s), and no row-wise likelihoods (want %d)",
 			 oo->name(), dataMat->getType(), expectation->numOrdinal,
-			 wantRowwiseLikelihood, expectation->name);
+			expectation->name, wantRowwiseLikelihood);
 	}
 
 	if (strEQ(omxDataType(dataMat), "raw")) {
@@ -350,7 +351,7 @@ omxFitFunction *MLFitState::initMorph()
 			omxRAMExpectation *ram = (omxRAMExpectation*) expectation;
 			if (ram->between.size()) {
 				if (useFellner == 0) {
-					Rf_error("%s: fellner=TRUE is required for %s",
+					mxThrow("%s: fellner=TRUE is required for %s",
 						 oo->name(), expectation->name);
 				}
 				useFellner = 1;
@@ -396,7 +397,7 @@ void MLFitState::init()
 
 	auto dc = oo->expectation->getDataColumns();
 	if (dc.size()) {
-		if (dataMat->isDynamic()) Rf_error("%s: dynamic data & column reordering"
+		if (dataMat->isDynamic()) mxThrow("%s: dynamic data & column reordering"
 						   " is not implemented yet", name());
 		newObj->copiedData = true;
 		newObj->observedCov = omxCreateCopyOfMatrix(newObj->observedCov, oo->matrix->currentState);
